/*
 * ${license.title}
 * Copyright (C) 2022 ${license.owner}
 * ${license.mailto}
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

//
// Created by webpigeon on 24/04/22.
//

#ifndef FGGL_PHYS_BULLET_BULLET_HPP
#define FGGL_PHYS_BULLET_BULLET_HPP

#include "fggl/ecs3/module/module.h"

#include "fggl/phys/types.hpp"
#include "fggl/phys/bullet/types.hpp"

namespace fggl::phys::bullet {

	/**
	 * Bullet integration module.
	 *
	 * This provides the ability for FGGL to use bullet as a physics backened. Bullet is mature, stable physics
	 * engine which makes it suitable for most use cases. For use with FGGL there is a reasonable amount of copying
	 * back and forth to keep the two states in sync.
	 */
	struct BulletModule : ecs3::Module {
		public:
			BulletModule() = default;

			[[nodiscard]]
			std::string name() const override {
				return "phys::Bullet3";
			}

			void onLoad(ecs3::ModuleManager &manager, ecs3::TypeRegistry &types) override {
				// dependencies
				types.make<phys::RigidBody>();

				// my types
				types.make<phys::bullet::BulletBody>();
			}

	};

} // namespace fggl::phys::bullet

namespace fggl::phys {

	// allows using fggl::phys::Bullet3 as the module name
	using Bullet3 = bullet::BulletModule;

} // namespace fggl::phys

#endif //FGGL_PHYS_BULLET_BULLET_HPP
